# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django.core.management.base import NoArgsCommand
from django.utils.timezone import now
from onpay.models import Order
import datetime


class Command(NoArgsCommand):
    def handle_noargs(self, **options):
        today = now()
        limit = today - datetime.timedelta(seconds=60 * 60)

        Order.objects.filter(
            date_created__lte=limit
        ).update(state=Order.STATE_EXPIRED)



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division


def get_version():
    return '0.2.1'



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from onpay.models import Order
from onpay.settings import ONPAY_CURRENCIES


def create_order(amount, currency, comment, email):
    if currency.upper() not in dict(ONPAY_CURRENCIES):
        raise TypeError('Unsupported currency code %s' % currency)
    return Order.objects.create(amount=amount,
                                currency=currency,
                                comment=comment,
                                email=email)



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from django.utils import six
from onpay import settings
from onpay.signals import (order_created, order_updated,
                           order_success, order_failure)
import hashlib


try:
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode


def md5(src):
    if six.PY3:
        src = src.encode('utf-8')
    return hashlib.md5(src).hexdigest().upper()


@python_2_unicode_compatible
class Order(models.Model):
    MODE_LIVE = 0
    MODE_TEST = 1

    STATE_SUCCESS = 0
    STATE_FAILURE = 1
    STATE_WAITING = 3
    STATE_EXPIRED = 2

    amount = models.IntegerField(_('amount'))
    currency = models.CharField(_('currency'),
                                max_length=3,
                                default=settings.ONPAY_CURRENCY,
                                choices=settings.ONPAY_CURRENCIES)
    comment = models.TextField(_('order notes'), default='')
    email = models.EmailField(_('buyer e-mail'))
    date_created = models.DateTimeField(_('created at'), auto_now_add=True)
    state = models.IntegerField(_('order status'),
                                default=STATE_WAITING,
                                choices=(
                                    (STATE_SUCCESS, _('Success')),
                                    (STATE_FAILURE, _('Failed')),
                                    (STATE_WAITING, _('Waiting')),
                                    (STATE_EXPIRED, _('Expired')),
                                    ))
    mode = models.IntegerField(_('mode'), default=MODE_LIVE,
                               choices=(
                                   (MODE_LIVE, _('live')),
                                   (MODE_TEST, _('text')),
                                   ))

    def mark_as_success(self):
        self._mark(state=self.STATE_SUCCESS, signal=order_success)

    def mark_as_failure(self):
        self._mark(state=self.STATE_FAILURE, signal=order_failure)

    def can_be_payed(self):
        return self.state == self.STATE_WAITING

    def is_successfully_paid(self):
        return self.state == self.STATE_SUCCESS

    def is_pay_failed(self):
        return self.state in [self.STATE_EXPIRED, self.STATE_FAILURE]

    def get_pay_url(self, currency=settings.ONPAY_CURRENCY,
                    success_url=settings.ONPAY_SUCCESS_URL,
                    failure_url=settings.ONPAY_SUCCESS_URL):
        pattern = 'fix;{amount:.1f};{currency};{order_id:d};no;{secret}'
        md5_str = pattern.format(amount=self.amount,
                                 currency=settings.ONPAY_CURRENCY,
                                 secret=settings.ONPAY_SECRET,
                                 order_id=self.id)
        if six.PY3:
            md5_str = md5_str.encode('utf-8')
        md5 = hashlib.md5(md5_str).hexdigest()

        return settings.ONPAY_GATE_URL + '?' + urlencode({
            'pay_mode': 'fix',
            'f': 7,
            'price': self.amount,
            'pay_for': self.id,
            'convert': 'no',
            'price_final': 'true',
            'user_email': self.email,
            'md5': md5,
            'currency': currency,
            'url_success_enc': success_url,
            'url_fail_enc': failure_url,
        })

    pay_url = property(get_pay_url)

    def _mark(self, state, signal):
        self.state = state
        self.save(update_fields=['state'])
        signal.send(sender=self.__class__, order=self)

    def _check(self, flag=''):
        pattern = 'check;{order_id};{amount:.1f};{currency};{flag}{secret}'
        raw_string = pattern.format(order_id=self.id,
                                    amount=self.amount,
                                    currency=settings.ONPAY_CURRENCY,
                                    secret=settings.ONPAY_SECRET,
                                    flag=flag)
        hashed_string = md5(raw_string)
        return hashed_string

    def crc_check_correct(self, crc):
        return crc == self.crc_check_get()

    def crc_check_get(self):
        return self._check(flag='')

    def crc_check_create(self):
        return self._check(flag='0;')

    def crc_pay_correct(self, crc, onpay_id, order_amount):
        actual_crc = self.crc_pay_get(onpay_id, order_amount)
        return crc == actual_crc

    def crc_pay_get(self, onpay_id, order_amount):
        pattern = ('pay;{order_id};{onpay_id};{order_amount};'
                   '{currency};{secret}')
        md5source = pattern.format(order_id=self.id,
                                   onpay_id=onpay_id,
                                   order_amount=order_amount,
                                   currency=settings.ONPAY_CURRENCY,
                                   secret=settings.ONPAY_SECRET)
        md5hash = md5(md5source)
        return md5hash

    def crc_pay_create(self, onpay_id, order_amount):
        pattern = ('pay;{po:d};{onpay_id};{order_id:d};{order_amount};'
                   '{currency};0;{secret}')
        result_m5source = pattern.format(po=self.id,
                                         onpay_id=onpay_id,
                                         order_id=self.id,
                                         order_amount=order_amount,
                                         currency=settings.ONPAY_CURRENCY,
                                         secret=settings.ONPAY_SECRET)
        result_m5hash = md5(result_m5source)
        return result_m5hash

    def __str__(self):
        return '#%s' % self.pk

    class Meta(object):
        verbose_name = _('payment')
        verbose_name_plural = _('payments')


def order_trigger(**kwargs):
    instance = kwargs['instance']
    created = kwargs['created']
    sender = kwargs['sender']

    if created:
        order_created.send(sender=sender, order=instance)
    else:
        order_updated.send(sender=sender, order=instance)


def mode_trigger(**kwargs):
    instance = kwargs['instance']
    if instance.pk:
        return
    if instance.currency == 'TST':
        instance.mode = instance.MODE_TEST


models.signals.pre_save.connect(mode_trigger, sender=Order,
                                dispatch_uid='onpay.models')
models.signals.post_save.connect(order_trigger, sender=Order,
                                 dispatch_uid='onpay.models')



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django.conf import settings as s
from django.utils.translation import ugettext_lazy as _


gate_url = 'https://secure.onpay.ru/pay/{gate}'


ONPAY_CURRENCY = getattr(s, 'ONPAY_CURRENCY', 'RUR')
ONPAY_CURRENCIES = getattr(s, 'ONPAY_CURRENCIES', (
    ('RUR', _('Russian rubles')),
    ('TST', _('Test currency')),
))
ONPAY_SECRET = getattr(s, 'ONPAY_SECRET', '****************')
ONPAY_GATE = getattr(s, 'ONPAY_GATE', 'login')
ONPAY_GATE_URL = getattr(s, 'ONPAY_GATE_URL', gate_url.format(gate=ONPAY_GATE))
ONPAY_SUCCESS_URL = getattr(s, 'ONPAY_SUCCESS_URL', '')
ONPAY_FAILURE_URL = getattr(s, 'ONPAY_SUCCESS_URL', '')



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django import test
from django.core.urlresolvers import reverse
from django.utils import six
from django.utils.timezone import now
from onpay.utils import create_order
from onpay import signals


# Some constants
ONPAY_ID = '123'
CRC_PAY_CORRECT = '5C89A599E3E15DE4E3EC2803DC0BF596'
CRC_PAY_CREATE = 'BEAC0AA488C13E9B89B5F8B8DA425F5C'
CRC_CHECK_CORRECT = '48CE93AD7E0053BA37316676F4B29C4A'
CRC_CHECK_CREATE = '8E1D15E585D02885B40BA7E69C7572FF'


def signal_catcher(*args, **kwargs):
    raise ZeroDivisionError


class TestViews(test.TestCase):
    urls = 'onpay.urls'

    def setUp(self):
        self.order = create_order(amount=10,
                                  currency='RUR',
                                  comment='Test order',
                                  email='bender@ilovebender.com')
        self.client = test.Client()
        self.url = reverse('onpay_result')

    def get_check_data(self):
        return {
            'type': 'check',
            'md5': CRC_CHECK_CORRECT,
            'order_currency': self.order.currency,
            'order_amount': 1.0,
            'amount': 1.0,
            'pay_for': self.order.pk
        }

    def get_pay_data(self):
        return {
            'type': 'pay',
            'md5': CRC_CHECK_CORRECT,
            'order_currency': self.order.currency,
            'order_amount': 1.0,
            'amount': 1.0,
            'pay_for': self.order.pk,
            'balance_amount': 1.0,
            'paid_amount': 1.0,
            'paymentDateTime': now().isoformat(),
            'onpay_id': ONPAY_ID,
            'balance_currency': self.order.currency,
            'exchange_rate': 1,
            'user_email': 'bender@ilovebender.com'
        }

    def assertOrderSuccessfullyPaid(self, order):
        self.assertTrue(
            order.__class__.objects.get(pk=order.pk).is_successfully_paid()
        )

    def assertOrderFailed(self, order):
        self.assertFalse(
            order.__class__.objects.get(pk=order.pk).is_successfully_paid()
        )

    def test_bad_request(self):
        resp = self.client.post(self.url)
        self.assertEquals(400, resp.status_code)

    def test_bad_request_pay_wrong_date(self):
        data = self.get_pay_data()
        data.update(paymentDateTime='not-a-timestamp')
        del data['type']  # wittingly invalid form
        resp = self.client.post(self.url, data=data)
        self.assertEquals(400, resp.status_code)

    def test_check_form_valid(self):
        resp = self.client.post(self.url, self.get_check_data())

        self.assertEquals(200, resp.status_code)
        self.assertContains(resp, '<comment>OK</comment>')
        self.assertContains(resp, CRC_CHECK_CREATE)

    def test_check_form_valid_md5_failed(self):
        data = self.get_check_data()
        data.update(md5='*****')
        resp = self.client.post(self.url, data=data)
        self.assertEquals(400, resp.status_code)

    def test_pay_form_valid(self):
        data = self.get_pay_data()
        data.update(md5=CRC_PAY_CORRECT)
        resp = self.client.post(self.url, data=data)

        self.assertEquals(200, resp.status_code)
        self.assertContains(resp, '<comment>OK</comment>')
        self.assertContains(resp, CRC_PAY_CREATE)
        self.assertOrderSuccessfullyPaid(self.order)

    def test_pay_form_valid_md5_failed(self):
        data = self.get_pay_data()
        data.update(md5='NOT_' + CRC_PAY_CORRECT)
        resp = self.client.post(self.url, data=data)
        self.assertEquals(400, resp.status_code)
        self.assertOrderFailed(self.order)


class TestUtils(test.TestCase):
    def test_create_order_full(self):
        order = create_order(amount=10.0,
                             currency='RUR',
                             comment='Test order',
                             email='bender@ilovebender.com')

        self.assertEquals(10.0, order.amount)
        self.assertEquals('RUR', order.currency)
        self.assertEquals('Test order', order.comment)
        self.assertEquals('bender@ilovebender.com', order.email)
        self.assertEquals(order.MODE_LIVE, order.mode)
        self.assertEquals(order.STATE_WAITING, order.state)

    def test_create_order_with_tst_currency(self):
        order = create_order(amount=10, currency='TST', comment='Test order',
                             email='bender@ilovebender.com')
        self.assertEquals('TST', order.currency)
        self.assertEquals(order.MODE_TEST, order.mode)

    def test_create_order_with_unknown_currency(self):
        def raised_exception():
            create_order(amount=10,
                         currency='WTF',
                         comment='Test order',
                         email='bender@ilovebender.com')
        self.assertRaises(TypeError, raised_exception)


class TestModels(test.TestCase):
    def create_order(self):
        return create_order(amount=10, currency='RUR',
                            comment='Test order',
                            email='bender@ilovebender.com')

    def assertSignalCatched(self, signal, callable_, ):
        signal.connect(signal_catcher)
        try:
            callable_()
            self.fail('Signal are not sent')
        except ZeroDivisionError:
            pass
        finally:
            signal.disconnect(signal_catcher)

    def setUp(self):
        self.order = self.create_order()

    def test_str(self):
        self.assertIn(container=six.text_type(self.order),
                      member=six.text_type(self.order.pk))

    def test_mark_as_success(self):
        self.order.mark_as_success()
        self.assertEquals(self.order.STATE_SUCCESS, self.order.state)

    def test_mark_as_success_signal(self):
        self.assertSignalCatched(signals.order_success,
                                 lambda: self.order.mark_as_success())

    def test_mark_as_failure(self):
        self.order.mark_as_failure()
        self.assertEquals(self.order.STATE_FAILURE,
                          self.order.state)

    def test_mark_as_failure_signal(self):
        self.assertSignalCatched(signals.order_failure,
                                 lambda: self.order.mark_as_failure())

    def test_can_be_payed(self):
        self.assertTrue(self.order.can_be_payed())

    def test_can_be_payed_not(self):
        order = self.order
        for state in [order.STATE_FAILURE,
                      order.STATE_SUCCESS,
                      order.STATE_EXPIRED]:
            order.state = state
            order.save()
            self.assertFalse(order.can_be_payed())

    def test_is_successfully_paid(self):
        order = self.order
        order.mark_as_success()
        self.assertTrue(order.is_successfully_paid())

    def test_is_successfully_paid_not(self):
        order = self.order
        for state in [order.STATE_FAILURE,
                      order.STATE_WAITING,
                      order.STATE_EXPIRED]:
            order.state = state
            order.save()
            self.assertFalse(order.is_successfully_paid())

    def test_is_pay_failed(self):
        order = self.order
        for state in [order.STATE_EXPIRED, order.STATE_FAILURE]:
            order.state = state
            order.save()
            self.assertTrue(order.is_pay_failed())

    def test_is_pay_failed_not(self):
        order = self.create_order()
        for state in [order.STATE_SUCCESS, order.STATE_WAITING]:
            order.state = state
            order.save()
            self.assertFalse(order.is_pay_failed())

    def test_get_pay_url_smoke(self):
        order = self.create_order()
        self.assertEquals(order.pay_url,
                          order.get_pay_url())

    def test_crc_check_correct(self):
        self.assertTrue(self.order.crc_check_correct(CRC_CHECK_CORRECT))

    def test_crc_check_correct_not(self):
        self.assertFalse(
            self.order.crc_check_correct('NOT_' + CRC_CHECK_CORRECT)
        )

    def test_crc_check_create(self):
        self.assertEquals(
            CRC_CHECK_CREATE, self.order.crc_check_create()
        )

    def test_crc_pay_correct(self):
        self.assertTrue(self.order.crc_pay_correct(crc=CRC_PAY_CORRECT,
                                                   onpay_id='123',
                                                   order_amount=1.0))

    def test_crc_pay_correct_not(self):
        self.assertFalse(
            self.order.crc_pay_correct(crc='NOT_' + CRC_PAY_CORRECT,
                                       onpay_id=ONPAY_ID,
                                       order_amount=1.0)
        )

    def test_crc_pay_get(self):
        self.assertEquals(CRC_PAY_CORRECT,
                          self.order.crc_pay_get(onpay_id=ONPAY_ID,
                                                 order_amount=1.0))

    def test_crc_pay_create(self):
        self.assertEquals(
            CRC_PAY_CREATE,
            self.order.crc_pay_create(onpay_id=ONPAY_ID,
                                      order_amount=1.0))



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django.contrib import admin
from onpay.models import Order


class OrderAdmin(admin.ModelAdmin):
    list_filter = ['state']
    date_hierarchy = 'date_created'
    list_display = ['id', 'comment', 'amount', 'email',
                    'date_created', 'state']

admin.site.register(Order, OrderAdmin)



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django.conf.urls import url
from onpay.views import result


urlpatterns = [
    url(r'^result/$', result, name='onpay_result'),
]



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django import forms
from onpay.models import Order
from dateutil.parser import parse


CURRENCY_CHOICES = (
    ('RUR', 'RUR'),
    ('TST', 'TST'),
)


class CheckForm(forms.Form):
    TYPE_CHOICES = (
        ('check', 'check'),
        ('pay', 'payment'),
    )
    type = forms.ChoiceField(choices=TYPE_CHOICES)
    md5 = forms.CharField()
    order_currency = forms.ChoiceField(choices=CURRENCY_CHOICES)
    order_amount = forms.FloatField()
    amount = forms.FloatField()
    pay_for = forms.ModelChoiceField(queryset=Order.objects.all())


class PayForm(CheckForm):
    balance_amount = forms.FloatField()
    paid_amount = forms.FloatField()
    paymentDateTime = forms.CharField()
    onpay_id = forms.IntegerField()
    user_phone = forms.CharField(required=False)
    balance_currency = forms.ChoiceField(choices=CURRENCY_CHOICES)
    note = forms.CharField(required=False)
    exchange_rate = forms.FloatField()
    protection_code = forms.CharField(required=False)
    day_to_expiry = forms.CharField(required=False)
    user_email = forms.EmailField()

    def clean_paymentDateTime(self):
        try:
            return parse(self.cleaned_data['paymentDateTime'])
        except (ValueError, TypeError):
            raise forms.ValidationError('Wrong paymentDateTime value')



#!/usr/bin/env python
# coding: utf-8

from setuptools import setup, find_packages
import os
import sys

py_version = sys.version_info
version = __import__('onpay').get_version()
readme = os.path.join(os.path.dirname(__file__), 'README.rst')

CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'Environment :: Web Environment',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Framework :: Django'
]

install_requires = ['django',
                    'python-dateutil',
                    'six']
tests_require = ['django',
                 'six']


if isinstance(py_version, tuple):
    if py_version < (2, 7):
        install_requires.append('importlib')


setup(
    name='django-onpay',
    author='Mikhail Porokhovnichenko <marazmiki@gmail.com>',
    version=version,
    author_email='marazmiki@gmail.com',
    url='http://pypi.python.org/pypi/django-onpay',
    download_url='http://bitbucket.org/marazmiki/django-onpay/get/tip.zip',
    description='onpay',
    long_description=open(readme).read(),
    license='MIT license',
    platforms=['OS Independent'],
    classifiers=CLASSIFIERS,
    install_requires=install_requires,
    packages=find_packages(exclude=['test_project', 'test_project.*']),
    test_suite='tests.main',
    tests_require=tests_require,
    include_package_data=True,
    zip_safe=False)



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django.dispatch import Signal


order_created = Signal(providing_args=['order'])
order_updated = Signal(providing_args=['order'])
order_success = Signal(providing_args=['order'])
order_failure = Signal(providing_args=['order'])



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django.http import HttpResponse, HttpResponseBadRequest
from django.template.loader import render_to_string
from django.views.decorators.csrf import csrf_exempt
from onpay.forms import CheckForm, PayForm
import logging


logger = logging.getLogger(__name__)


@csrf_exempt
def result(request):
    check_form = CheckForm(request.POST or None)
    pay_form = PayForm(request.POST or None)

    if not check_form.is_valid() and not pay_form.is_valid():
        return HttpResponseBadRequest('Wrong API request')

    if pay_form.is_valid():
        po = pay_form.cleaned_data['pay_for']
        onpay_id = pay_form.cleaned_data['onpay_id']
        order_amount = pay_form.cleaned_data['order_amount']

        if po.crc_pay_correct(crc=pay_form.cleaned_data['md5'],
                              order_amount=order_amount,
                              onpay_id=onpay_id):

            md5_checksum = po.crc_pay_create(order_amount=order_amount,
                                             onpay_id=onpay_id)

            result_xml = render_to_string('onpay/pay_success.xml',
                                          {'onpay_id': onpay_id,
                                           'pay_for': po.id,
                                           'order_id': po.id,
                                           'md5_checksum': md5_checksum})
            po.mark_as_success()
            return HttpResponse(result_xml, content_type='text/xml')

        else:
            po.mark_as_failure()

    if check_form.is_valid():
        order = check_form.cleaned_data['pay_for']

        if order.crc_check_correct(check_form.cleaned_data['md5']):
            md5_checksum = order.crc_check_create()
            result_xml = render_to_string('onpay/check_success.xml',
                                          {'md5_checksum': md5_checksum,
                                           'pay_for': order.id})
            return HttpResponse(result_xml, content_type='text/xml')

        else:
            return HttpResponseBadRequest('Check failed')
